@@ -332,7 +332,7 @@ void FourSuite::absolutize_test_cases() {
 	// 82-88
 	TEST_ASSERT(testAddOrRemoveBaseHelper("http:this", "http://example.org/base/uri", "http:this"));
 	TEST_ASSERT(testAddOrRemoveBaseHelper("http:this", "http:base", "http:this"));
-	// TODO Whole in the URI spec, see http://lists.w3.org/Archives/Public/uri/2007Aug/0003.html
+	// Whole in the URI spec, see http://lists.w3.org/Archives/Public/uri/2007Aug/0003.html
 	// TEST_ASSERT(testAddOrRemoveBaseHelper(".//g", "f:/a", "f://g")); // ORIGINAL
 	TEST_ASSERT(testAddOrRemoveBaseHelper(".//g", "f:/a", "f:/.//g")); // FIXED ONE
 	TEST_ASSERT(testAddOrRemoveBaseHelper("b/c//d/e", "f://example.org/base/a", "f://example.org/base/b/c//d/e"));
@@ -593,9 +593,15 @@ void FourSuite::pctEncNormalizationTests() {
 
 void FourSuite::pathSegmentNormalizationTests() {
 	TEST_ASSERT(normalizeAndCompare("/a/b/../../c", "/c"));
-	TEST_ASSERT(normalizeAndCompare("a/b/../../c", "a/b/../../c"));
+	// TEST_ASSERT(normalizeAndCompare("a/b/../../c", "a/b/../../c"));
+	// Fixed:
+	TEST_ASSERT(normalizeAndCompare("a/b/../../c", "c"));
 	TEST_ASSERT(normalizeAndCompare("/a/b/././c", "/a/b/c"));
-	TEST_ASSERT(normalizeAndCompare("a/b/././c", "a/b/././c"));
+	// TEST_ASSERT(normalizeAndCompare("a/b/././c", "a/b/././c"));
+	// Fixed:
+	TEST_ASSERT(normalizeAndCompare("a/b/././c", "a/b/c"));
 	TEST_ASSERT(normalizeAndCompare("/a/b/../c/././d", "/a/c/d"));
-	TEST_ASSERT(normalizeAndCompare("a/b/../c/././d", "a/b/../c/././d"));
+	// TEST_ASSERT(normalizeAndCompare("a/b/../c/././d", "a/b/../c/././d"));
+	// Fixed:
+	TEST_ASSERT(normalizeAndCompare("a/b/../c/././d", "a/c/d"));
 }
