@@ -67,7 +67,7 @@ public:
 		TEST_ADD(UriSuite::testUriHostIpSix2)
 		TEST_ADD(UriSuite::testUriHostIpFuture)
 		TEST_ADD(UriSuite::testUriComponents)
-		TEST_ADD(UriSuite::testUriComponentsBug20070701)
+		TEST_ADD(UriSuite::testUriComponents_Bug20070701)
 		TEST_ADD(UriSuite::testEscaping)
 		TEST_ADD(UriSuite::testUnescaping)
 		TEST_ADD(UriSuite::testTrailingSlash)
@@ -77,7 +77,11 @@ public:
 		TEST_ADD(UriSuite::testToStringCharsRequired)
 		TEST_ADD(UriSuite::testNormalizeSyntaxMaskRequired)
 		TEST_ADD(UriSuite::testNormalizeSyntax)
+		TEST_ADD(UriSuite::testNormalizeSyntaxComponents)
+		TEST_ADD(UriSuite::testNormalizeCrash_Bug20080224)
 		TEST_ADD(UriSuite::testFilenameUriConversion)
+		TEST_ADD(UriSuite::testCrash_FreeUriMembers_Bug20080116)
+		TEST_ADD(UriSuite::testCrash_MakeOwner_Bug20080207)
 	}
 
 private:
@@ -301,7 +305,7 @@ private:
 		uriFreeUriMembersA(&uriA);
 	}
 
-	void testUriComponentsBug20070701() {
+	void testUriComponents_Bug20070701() {
 		UriParserStateA stateA;
 		UriUriA uriA;
 		stateA.uri = &uriA;
@@ -1006,7 +1010,8 @@ private:
 		TEST_ASSERT(testNormalizeMaskHelper(L"http://localhost/#AB%43", URI_NORMALIZE_FRAGMENT));
 	}
 
-	bool testNormalizeSyntaxHelper(wchar_t * uriText, wchar_t * expectedNormalized) {
+	bool testNormalizeSyntaxHelper(wchar_t * uriText, wchar_t * expectedNormalized,
+			unsigned int mask = static_cast<unsigned int>(-1)) {
 		UriParserStateW stateW;
 		int res;
 
@@ -1028,21 +1033,27 @@ private:
 			return false;
 		}
 
-		const bool equalBefore = (URI_TRUE == uriEqualsUriW(&testUri, &expectedUri));
-		if (equalBefore) {
-			uriFreeUriMembersW(&testUri);
-			uriFreeUriMembersW(&expectedUri);
-			return false;
-		}
-
-		res = uriNormalizeSyntaxW(&testUri);
+		// First run
+		res = uriNormalizeSyntaxExW(&testUri, mask);
 		if (res != 0) {
 			uriFreeUriMembersW(&testUri);
 			uriFreeUriMembersW(&expectedUri);
 			return false;
 		}
 
-		const bool equalAfter = (URI_TRUE == uriEqualsUriW(&testUri, &expectedUri));
+		bool equalAfter = (URI_TRUE == uriEqualsUriW(&testUri, &expectedUri));
+
+		// Second run
+		res = uriNormalizeSyntaxExW(&testUri, mask);
+		if (res != 0) {
+			uriFreeUriMembersW(&testUri);
+			uriFreeUriMembersW(&expectedUri);
+			return false;
+		}
+
+		equalAfter = equalAfter
+				&& (URI_TRUE == uriEqualsUriW(&testUri, &expectedUri));
+
 		uriFreeUriMembersW(&testUri);
 		uriFreeUriMembersW(&expectedUri);
 		return equalAfter;
@@ -1052,6 +1063,115 @@ private:
 		TEST_ASSERT(testNormalizeSyntaxHelper(
 				L"eXAMPLE://a/./b/../b/%63/%7bfoo%7d",
 				L"example://a/b/c/%7Bfoo%7D"));
+
+		// Testcase by Adrian Manrique
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"http://examp%4Ce.com/",
+				L"http://example.com/"));
+
+		// Testcase by Adrian Manrique
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"http://example.com/a/b/%2E%2E/",
+				L"http://example.com/a/"));
+
+		// Reported by Adrian Manrique
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"http://user:pass@SOMEHOST.COM:123",
+				L"http://user:pass@somehost.com:123"));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"HTTP://a:b@HOST:123/./1/2/../%41?abc#def",
+				L"http://a:b@host:123/1/A?abc#def"));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"../../abc",
+				L"../../abc"));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"../../abc/..",
+				L"../../"));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"../../abc/../def",
+				L"../../def"));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"abc/..",
+				L""));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"abc/../",
+				L""));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"../../abc/./def",
+				L"../../abc/def"));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"./def",
+				L"def"));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"def/.",
+				L"def/"));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"./abc:def",
+				L"./abc:def"));
+	}
+
+	void testNormalizeSyntaxComponents() {
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"HTTP://%41@EXAMPLE.ORG/../a?%41#%41",
+				L"http://%41@EXAMPLE.ORG/../a?%41#%41",
+				URI_NORMALIZE_SCHEME));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"HTTP://%41@EXAMPLE.ORG/../a?%41#%41",
+				L"HTTP://A@EXAMPLE.ORG/../a?%41#%41",
+				URI_NORMALIZE_USER_INFO));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"HTTP://%41@EXAMPLE.ORG/../a?%41#%41",
+				L"HTTP://%41@example.org/../a?%41#%41",
+				URI_NORMALIZE_HOST));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"HTTP://%41@EXAMPLE.ORG/../a?%41#%41",
+				L"HTTP://%41@EXAMPLE.ORG/a?%41#%41",
+				URI_NORMALIZE_PATH));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"HTTP://%41@EXAMPLE.ORG/../a?%41#%41",
+				L"HTTP://%41@EXAMPLE.ORG/../a?A#%41",
+				URI_NORMALIZE_QUERY));
+
+		TEST_ASSERT(testNormalizeSyntaxHelper(
+				L"HTTP://%41@EXAMPLE.ORG/../a?%41#%41",
+				L"HTTP://%41@EXAMPLE.ORG/../a?%41#A",
+				URI_NORMALIZE_FRAGMENT));
+	}
+
+	void testNormalizeCrash_Bug20080224() {
+		UriParserStateW stateW;
+		int res;
+		UriUriW testUri;
+		stateW.uri = &testUri;
+
+		res = uriParseUriW(&stateW, L"http://example.org/abc//../def");
+		TEST_ASSERT(res == 0);
+
+		// First call will make us owner of copied memory
+		res = uriNormalizeSyntaxExW(&testUri, URI_NORMALIZE_SCHEME);
+		TEST_ASSERT(res == 0);
+		res = uriNormalizeSyntaxExW(&testUri, URI_NORMALIZE_HOST);
+		TEST_ASSERT(res == 0);
+
+		// Frees empty path segment -> crash
+		res = uriNormalizeSyntaxW(&testUri);
+		TEST_ASSERT(res == 0);
+
+		uriFreeUriMembersW(&testUri);
 	}
 
 	void testFilenameUriConversionHelper(const wchar_t * filename,
@@ -1066,17 +1186,19 @@ private:
 		} else {
 			uriWindowsFilenameToUriStringW(filename, uriBuffer);
 		}
+		// wprintf(L"1 [%s][%s]\n", uriBuffer, uriString);
 		TEST_ASSERT(!wcscmp(uriBuffer, uriString));
 		delete [] uriBuffer;
 
 		// URI string to filename
-		const size_t filenameBufferLen = wcslen(uriString) + 1 - prefixLen;
+		const size_t filenameBufferLen = wcslen(uriString) + 1;
 		wchar_t * filenameBuffer = new wchar_t[filenameBufferLen];
 		if (forUnix) {
 			uriUriStringToUnixFilenameW(uriString, filenameBuffer);
 		} else {
 			uriUriStringToWindowsFilenameW(uriString, filenameBuffer);
 		}
+		// wprintf(L"2 [%s][%s]\n", filenameBuffer, filename);
 		TEST_ASSERT(!wcscmp(filenameBuffer, filename));
 		delete [] filenameBuffer;
 	}
@@ -1085,7 +1207,44 @@ private:
 		const bool FOR_UNIX = true;
 		const bool FOR_WINDOWS = false;
 		testFilenameUriConversionHelper(L"/bin/bash", L"file:///bin/bash", FOR_UNIX);
+		testFilenameUriConversionHelper(L"./configure", L"./configure", FOR_UNIX);
+
 		testFilenameUriConversionHelper(L"E:\\Documents and Settings", L"file:///E:/Documents%20and%20Settings", FOR_WINDOWS);
+		testFilenameUriConversionHelper(L".\\Readme.txt", L"./Readme.txt", FOR_WINDOWS);
+
+		testFilenameUriConversionHelper(L"index.htm", L"index.htm", FOR_WINDOWS);
+		testFilenameUriConversionHelper(L"index.htm", L"index.htm", FOR_UNIX);
+
+		testFilenameUriConversionHelper(L"abc def", L"abc%20def", FOR_WINDOWS);
+		testFilenameUriConversionHelper(L"abc def", L"abc%20def", FOR_UNIX);
+	}
+
+	void testCrash_FreeUriMembers_Bug20080116() {
+		// Testcase by Adrian Manrique
+		UriParserStateA state;
+		UriUriA uri;
+		state.uri = &uri;
+		uriParseUriA(&state, "http://test/?");
+		uriNormalizeSyntaxA(&uri);
+		uriFreeUriMembersA(&uri);
+
+		TEST_ASSERT(true);
+	}
+
+	void testCrash_MakeOwner_Bug20080207() {
+		// Testcase by Adrian Manrique
+		UriParserStateA state;
+		UriUriA sourceUri;
+		state.uri = &sourceUri;
+		const char * const sourceUriString = "http://user:pass@somehost.com:80/";
+		if (uriParseUriA(&state, sourceUriString) != 0) {
+			TEST_ASSERT(false);
+		}
+		if (uriNormalizeSyntaxA(&sourceUri) != 0) {
+			TEST_ASSERT(false);
+		}
+		uriFreeUriMembersA(&sourceUri);
+		TEST_ASSERT(true);
 	}
 
 };
